"use strict";

var _set2 = _interopRequireDefault(require("lodash/set"));

var _chalk = _interopRequireDefault(require("chalk"));

var _child_process = require("child_process");

var _ejs = _interopRequireDefault(require("ejs"));

var _fsExtra = _interopRequireDefault(require("fs-extra"));

var _path = _interopRequireDefault(require("path"));

var _yargs = _interopRequireDefault(require("yargs"));

var _prompt = _interopRequireDefault(require("./helpers/prompt"));

var _summaryMessage = _interopRequireDefault(require("./helpers/summaryMessage"));

var _packages = _interopRequireDefault(require("./packages"));

var _types = require("./types");

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

const COMMON_FILES = _path.default.resolve(__dirname, '../templates/common');

const GRAPHQL_FILES = _path.default.resolve(__dirname, '../templates/graphql');

const REDUX_FILES = _path.default.resolve(__dirname, '../templates/redux');

const NAVIGATION_FILES = _path.default.resolve(__dirname, '../templates/navigation'); // const FASTLANE_FILES = path.resolve(__dirname, '../templates/fastlane');


const args = {
  integrations: {
    description: 'Which integrations/features would you like to have in your app?',
    choices: [_types.Integrations.Fastlane, _types.Integrations.GraphQL, _types.Integrations.Redux, _types.Integrations.ExpoModules]
  },
  modules: {
    description: 'Which packages would you like to have in your app?',
    choices: [_types.Modules.Config, _types.Modules.GestureHandler, _types.Modules.Navigation, _types.Modules.Reanimated, _types.Modules.Screens, _types.Modules.Svg, _types.Modules.VectorIcons]
  }
};

async function create(argv) {
  const startTime = process.hrtime();

  const folder = _path.default.join(process.cwd(), argv.name);

  const basename = _path.default.basename(argv.name);

  const questions = {
    integrations: {
      type: 'multiselect',
      name: 'integrations',
      message: 'Which integrations/features would you like to have in your app?',
      choices: [{
        title: 'Redux',
        value: _types.Integrations.Redux
      }, {
        title: 'GraphQL',
        value: _types.Integrations.GraphQL
      }, {
        title: 'ExpoModules',
        value: _types.Integrations.ExpoModules
      } // { title: 'Fastlane', value: Integrations.Fastlane },
      // { title: 'Sentry', value: Integrations.Sentry },
      ]
    },
    modules: {
      type: 'multiselect',
      name: 'modules',
      message: 'Which packages would you like to add to your app?',
      choices: [{
        title: 'react-native-screens',
        value: _types.Modules.Screens
      }, {
        title: 'react-native-reanimated',
        value: _types.Modules.Reanimated
      }, {
        title: 'react-native-gesture-handler',
        value: _types.Modules.GestureHandler
      }, {
        title: 'react-native-svg',
        value: _types.Modules.Svg
      }, {
        title: 'react-native-vector-icons',
        value: _types.Modules.VectorIcons
      }, {
        title: 'react-navigation',
        value: _types.Modules.Navigation
      } // { title: 'react-native-config', value: Modules.Config },
      ]
    }
  };
  const extraOptionsQuestions = {
    modules: {
      [_types.Modules.Navigation]: [{
        type: 'select',
        name: 'variant',
        message: 'Which starting navigation would you like to add to your app?',
        choices: [{
          title: 'Stack navigation',
          value: 'stack'
        }, {
          title: 'Native Stack navigation',
          value: 'nativeStack'
        }, {
          title: 'Bottom Bar navigation',
          value: 'bottomBar'
        }]
      }]
    }
  };
  const {
    integrations = [],
    modules = []
  } = { ...argv,
    ...(await (0, _prompt.default)(Object.entries(questions).filter(([k, v]) => !(argv[k] && v.validate ? v.validate(argv[k]) === true : Boolean(argv[k]))).map(([, v]) => v)))
  };

  const hasModuleSelected = moduleName => [...integrations, ...modules].includes(moduleName);

  const flatQuestions = Object.entries(extraOptionsQuestions).flatMap(([answerKey, questionsPerAnswer]) => Object.entries(questionsPerAnswer).filter(([moduleName]) => hasModuleSelected(moduleName)).flatMap(([moduleName, moduleQuestions]) => moduleQuestions.flatMap(({
    name,
    ...moduleQuestion
  }) => ({ ...moduleQuestion,
    name: `${answerKey}.${moduleName}.${name}`
  }))));
  const extraOptions = Object.entries(await (0, _prompt.default)(flatQuestions)).reduce((acc, [k, v]) => (0, _set2.default)(acc, k, v), {});

  if (!(await _fsExtra.default.pathExists(folder))) {
    console.log(_chalk.default.bgCyan.black(`Initialising react-native project ${basename}. It may take some time...`));
    (0, _child_process.execSync)('npx react-native init ' + basename + ' --template react-native-template-typescript --skip-install', {
      stdio: 'inherit'
    });
    console.log(_chalk.default.green('✅ Project initialised'));
  }
  /**
   * Creating options for ejs parser i.e:
   * { integrations: { redux: true }, modules: { screens: true } }
   * TODO: Maybe it's overcomplicated. Get back to it after some time :)
   */


  const options = Object.entries({
    integrations,
    modules
  }).reduce((options, [mainKey, values]) => {
    options[mainKey] = values.reduce((acc, key) => {
      acc[key] = true;
      return acc;
    }, {});
    return options;
  }, {
    project: {
      name: basename,
      package: basename.toLowerCase()
    },
    extraOptions
  });

  if (options.modules.navigation) {
    options.modules.gestureHandler = true;
    options.modules.reanimated = true;
    options.modules.screens = true;
  }

  const copyDir = async (source, dest) => {
    let processedFile;

    try {
      await _fsExtra.default.mkdirp(dest);
      const files = await _fsExtra.default.readdir(source);

      for (const f of files) {
        processedFile = f;

        const target = _path.default.join(dest, _ejs.default.render(f.replace(/^\$/, '').replace('.ejs', ''), options, {
          openDelimiter: '{',
          closeDelimiter: '}'
        }));

        const file = _path.default.join(source, f);

        const stats = await _fsExtra.default.stat(file);

        if (stats.isDirectory()) {
          await copyDir(file, target);
        } else {
          const content = await _fsExtra.default.readFile(file, 'utf8');
          const renderedFile = await _ejs.default.render(content, options, {
            async: true
          });
          await _fsExtra.default.writeFile(target, renderedFile);
        }
      }
    } catch (error) {
      console.log(_chalk.default.red(`Error ${processedFile}:`, error));
    }
  };

  const [packages, devPackages] = (0, _packages.default)(options);
  console.log(_chalk.default.blueBright('Copying common files'));
  await copyDir(COMMON_FILES, folder);
  console.log(_chalk.default.green('✅ Copied common files'));

  if (options.integrations.graphql) {
    console.log(_chalk.default.blueBright('Copying GraphQL files'));
    await copyDir(GRAPHQL_FILES, folder);
    console.log(_chalk.default.green('✅ Copied GraphQL files'));
  }

  if (options.integrations.redux) {
    console.log(_chalk.default.blueBright('Copying Redux files'));
    await copyDir(REDUX_FILES, folder);
    console.log(_chalk.default.green('✅ Copied Redux files'));
  } // if (options.integrations.fastlane) {
  //   console.log(chalk.blueBright('Copying Fastlane files'));
  //   await copyDir(FASTLANE_FILES, folder);
  //   console.log(chalk.green('✅ Copied Fastlane files'));
  // }


  if (options.modules.navigation) {
    console.log(_chalk.default.blueBright('Copying react-navigation files'));
    await copyDir(NAVIGATION_FILES, folder);
    console.log(_chalk.default.green('✅ Copied react-navigation files'));
  }
  /**
   * A bit hackish but for now I did not found anything better :shrug:
   */


  (0, _child_process.execSync)('rm -rf ' + folder + '/__tests__', {
    stdio: 'inherit'
  });
  (0, _child_process.execSync)('chmod +x ' + folder + '/scripts/postinstall.sh', {
    stdio: 'inherit'
  });

  if (packages.length > 0) {
    console.log(_chalk.default.blueBright(`Adding dependencies ${packages.join(', ')}`));
    (0, _child_process.execSync)('yarn --cwd ' + basename + ' add --ignore-scripts ' + packages.join(' '), {
      stdio: 'inherit'
    });
    console.log(_chalk.default.green('✅ Dependencies added'));
  }

  if (devPackages.length > 0) {
    console.log(_chalk.default.blueBright(`Adding dev dependencies ${devPackages.join(', ')}`));
    (0, _child_process.execSync)('yarn --cwd ' + basename + ' add --dev --ignore-scripts ' + devPackages.join(' '), {
      stdio: 'inherit'
    });
    console.log(_chalk.default.green('✅ Dev dependencies added'));
  }

  console.log(_chalk.default.blueBright(`Installing dependencies...`));
  (0, _child_process.execSync)('yarn --cwd ' + basename + ' install --ignore-scripts', {
    stdio: 'inherit'
  });
  console.log(_chalk.default.green('✅ Dependencies installed'));
  console.log(_chalk.default.blueBright(`Installing Pods...`));
  (0, _child_process.execSync)('yarn --cwd ' + basename + ' install', {
    stdio: 'inherit'
  });
  console.log(_chalk.default.green('✅ Pods installed'));
  console.log(_chalk.default.blueBright(`Linting...`));
  (0, _child_process.execSync)('yarn --cwd ' + basename + ' lint --fix', {
    stdio: 'inherit'
  });
  console.log(_chalk.default.green('✅ Lint successful'));
  console.log((0, _summaryMessage.default)(basename));
  const [durationInSeconds] = process.hrtime(startTime);
  console.log(_chalk.default.cyanBright(`✅ Project initialised in ${parseInt(`${durationInSeconds / 60}`)}min ${durationInSeconds % 60}s`));
}

_yargs.default.command('$0 <name>', 'create a react native library', args, create).demandCommand().recommendCommands().strict().argv;
//# sourceMappingURL=index.js.map