"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = void 0;

var _get2 = _interopRequireDefault(require("lodash/get"));

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

const PACKAGES = {
  common: {
    dev: ['@types/jest', '@types/lodash', '@types/react', '@types/react-native', '@types/react-test-renderer', 'babel-jest', 'babel-plugin-i18next-extract', 'babel-plugin-lodash', 'babel-plugin-module-resolver', 'babel-plugin-transform-remove-console', 'eslint', 'eslint-config-dsznajder', 'husky', 'jest', 'lint-staged', 'prettier', 'typescript'],
    main: ['react-i18next', 'i18next']
  },
  integrations: {
    graphql: {
      main: ['@apollo/client', 'graphql'],
      dev: ['@graphql-codegen/cli', '@graphql-codegen/introspection', '@graphql-codegen/near-operation-file-preset', '@graphql-codegen/typescript', '@graphql-codegen/typescript-operations', 'unfetch']
    },
    expoModules: {
      main: ['expo', 'expo-application']
    },
    fastlane: {
      main: []
    },
    sentry: {
      main: ['@sentry/react-native']
    },
    redux: {
      main: ['@reduxjs/toolkit', 'react-redux', 'redux'],
      dev: ['@types/react-redux', 'redux-flipper']
    }
  },
  modules: {
    vectorIcons: {
      main: ['react-native-vector-icons']
    },
    svg: {
      main: ['react-native-svg']
    },
    config: {
      main: ['react-native-config']
    },
    gestureHandler: {
      main: ['react-native-gesture-handler']
    },
    navigation: {
      main: ['@react-native-community/masked-view', '@react-navigation/native', 'react-native-safe-area-context']
    },
    reanimated: {
      main: ['react-native-reanimated', 'react-native-redash']
    },
    screens: {
      main: ['react-native-screens']
    }
  },
  extraOptions: {
    modules: {
      navigation: {
        variant: {
          bottomBar: {
            main: ['@react-navigation/stack', '@react-navigation/bottom-tabs']
          },
          stack: {
            main: ['@react-navigation/stack']
          },
          nativeStack: {
            main: []
          }
        }
      }
    }
  }
};

const getPackagesToInstall = options => {
  let packages = PACKAGES.common.main;
  let devPackages = PACKAGES.common.dev;

  const addPackages = ({
    main,
    dev
  }) => {
    packages.push(...main);
    devPackages.push(...(dev || []));
  };

  Object.entries(options.integrations).forEach(([key, value]) => {
    if (value) addPackages(PACKAGES.integrations[key]);
  });
  Object.entries(options.modules).forEach(([key, value]) => {
    if (value) addPackages(PACKAGES.modules[key]);
  });
  Object.entries(options.extraOptions).forEach(([optionKey, optionNames]) => Object.entries(optionNames).forEach(([optionName, optionValue]) => Object.entries(optionValue).forEach(([extraOptionKey, value]) => {
    const packagesForExtraOption = (0, _get2.default)(PACKAGES.extraOptions, [optionKey, optionName, extraOptionKey, value]);

    if (packagesForExtraOption) {
      addPackages(packagesForExtraOption);
    }
  })));
  return [packages, devPackages];
};

var _default = getPackagesToInstall;
exports.default = _default;
//# sourceMappingURL=packages.js.map